/**
 * Calculator of ariphmetic expressions
 * Uses CUP (Java version of YACC) technology
 */
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

//
//-----------------------------------------------------------
import Der;
//-----------------------------------------------------------
//

import java_cup.runtime.*;  // CUP runtime environment
import parser;  // Parser generated by CUP from the file "CUPCalc.cup"

public class CUPCalc extends Applet {
    Button evaluateButton;
    TextField expression;
    TextField result;

    // Layout constants
    static final int margin         = 5;
    static final int headlineHeight = 30;
    static final int vertSkip       = 5;
    static final int horSkip        = 5;
    static final int labelWidth     = 80;
    static final int textWidth      = 300;
    static final int buttonWidth    = 80;
    static final int buttonHeight   = 30;

    public static void main(String[] args) {
        Frame f = new Frame("CUP Calculator");

        WindowAdapter wa = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        };
        f.addWindowListener(wa);

        f.setFont(new Font("Helvetica", Font.PLAIN, 14));

        // Add menu bar
        final MenuBar mb = new MenuBar();
        f.setMenuBar(mb);
        final Menu fileMenu = new Menu("File");
        final MenuItem quitItem = new MenuItem("Quit");
        fileMenu.add(quitItem);
        mb.add(fileMenu);

        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // System.out.println("Action " + e);
                if (e.getActionCommand().equals("Quit")) {
                    System.exit(0);
                }
            }
        };
        fileMenu.addActionListener(al);

        CUPCalc cc = new CUPCalc();
        f.add("Center", cc);

        // f.pack();
        f.setSize(
            2*margin + labelWidth + horSkip + textWidth + 10,
            2*margin + headlineHeight + 4*buttonHeight + 3*vertSkip + 50
        );
        cc.init();
        cc.start();

        f.setVisible(true);
    }

    public void init() {
        setLayout(null);

        setFont(new Font("Helvetica", Font.PLAIN, 14));
        setBackground(Color.lightGray);

//
//-------------------------------------------------------------
        Label headline = new Label("Computing the derivation of algebraic function");
//-------------------------------------------------------------
//
        headline.setFont(new Font("TimesRoman", Font.BOLD, 18));
        add(headline);

        int x = margin;
        int y = margin;
        headline.setBounds(x, y, 380, 30);

        y += headlineHeight + vertSkip;

        Label l = new Label("Expression:");
        add(l);
        l.setBounds(x, y, labelWidth, buttonHeight);
        x += labelWidth + horSkip;

        expression = new TextField();
        add(expression);
        expression.setBounds(x, y, textWidth, buttonHeight);
        y += buttonHeight + vertSkip;

        evaluateButton = new Button("Derivate");
        add(evaluateButton);
        x += (textWidth - buttonWidth);
        evaluateButton.setBounds(x, y, buttonWidth, buttonHeight);
        y += buttonHeight + vertSkip;

        x = margin;

        l = new Label("Result:");
        add(l);
        l.setBounds(x, y, labelWidth, buttonHeight);
        x += labelWidth + horSkip;

        result = new TextField();
        result.setEditable(false);
        add(result);
        result.setBounds(x, y, textWidth, buttonHeight);
        result.setForeground(Color.blue);

        // Add action listener for "Expression" text field
        // (for processing of the "Enter" button) and "Evaluate" button
        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // System.out.println("Text action " + e);
                onEvaluate();
            }
        };

        expression.addActionListener(al);
        evaluateButton.addActionListener(al);
        // That's all.
    }

    // Evaluate an expression
    void onEvaluate() {
        parser parser_obj = new parser();
        parser_obj.setParsingLine(expression.getText());

        try {

            // Parse a line
            Symbol s = parser_obj.parse();
//
//-----------------------------------------------------  
            if (s != null && s.value != null) {
                result.setText(Der.derivation("" + s.value));
            } else {
                result.setText("Syntax error");
            }
//-----------------------------------------------------
//

        } catch (Exception e) {
            //... System.err.println("" + e);
            result.setText("Syntax error");
        }
    }
}
